<?php
require_once('request.php');
require_once('response.php');
require_once('error_response.php');

require_once('op5/auth/Auth.php');

require_once ('op5/license/op5lic.php');
require_once ('op5/sysinfo.php');

$nacoma_path = '/opt/monitor/op5/nacoma/';
define('NINJA_PATH', '/opt/monitor/op5/ninja/');
$request = Api_Request::factory();

/**
 * Setup kohana ('s autoload, really) so that we're not completely
 * without a DAL and helper functions.
 */
function load_kohana() {
	if(!defined('SKIP_KOHANA')) {
		define('SKIP_KOHANA', true);
		require_once rtrim(NINJA_PATH, '/').'/index.php';
	}
}

/**
 * @param $request Api_Request
 * @throws Api_Error_Response
 * @return Op5Auth
 */
function auth(Api_Request $request) {
	$auth_method = $request->get_param('auth_method', 'basic_auth');
	$conf = Op5Config::instance()->getConfig('http_api');
	if(!isset($conf['auth_method'][$auth_method])) {
		throw new Api_Error_Response('Invalid request', 400, "Bad value for 'auth_method' parameter ($auth_method), ".var_export($conf, true));
	}
	if(!$conf['auth_method'][$auth_method]) {
		// safety check: don't spawn more basic auth windows if it's impossible
		// to enter a combination to actually pass the auth; ergo, tell the user
		// that the API is offline
		foreach($conf['auth_method'] as $enabled) {
			if($enabled) {
				throw new Api_Error_Response('Invalid request', 412, "Given 'auth_method' ($auth_method) is disabled through http_api.yml, enable it or use another 'auth_method'");
			}
		}
		throw new Api_Error_Response('Forbidden', 403, "The API is locked down, all 'auth_method's are disabled through http_api.yml");
	}
	switch($auth_method) {
	case 'ninja':
		load_kohana();
		// must call csrf's token() to set it up
		csrf::token();
		$auth = Op5Auth::factory();
		if($auth->get_user() instanceof op5User_NoAuth) {
			throw new Api_Error_Response('Unauthorized', 401, "You need to login to access this page. You tried to reuse the Monitor session but you seem to be logged out of Monitor");
		}
		if(!csrf::valid($param = $request->get_param('csrf_token'))) {
			throw new Api_Error_Response('Unauthorized', 401, 'Invalid CSRF token provided. To generate a new token you need to log out of monitor and log in again');
		}
		break;
	case 'basic_auth':
		/* Fetch auth object without session support */
		$auth = Op5Auth::factory(array('session_key' => false));

		/* Check for an apache user */
		if ($auth->get_user() instanceof op5User_NoAuth) {
			/* Login every time - not to keep session */
			if (!isset($_SERVER['PHP_AUTH_USER']) ||
				!isset($_SERVER['PHP_AUTH_PW']) ||
				!$auth->login($_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW']))
			{
				throw new Api_Error_Response('Unauthorized', 401, 'You need to login to access this page');
			}
		}
		break;
	default:
		// only happens if there's a method specified in http_api.yml but there's no logic
		// above for that case. add that logic
		throw new Api_Error_Response('Invalid request', 400, "Bad value for 'auth_method' parameter");
	}
	return $auth;
}

/**
 * The license should be valid for the user to access the API.
 *
 * @return string error message
 */
function license() {
	$lic = new op5lic();
	if($lic->fetch_property('unlicensed')) {
		// we're not really sure why, since we didn't query() the
		// license with a specific metric/property
		return 'Your license is invalid';
	}
	return '';
}

try {
	$auth = auth($request);
	$license_error = license();
	if($license_error) {
		op5log::instance('http_api')->log('notice', sprintf("Logged in successfully as %s but aborting due to invalid license: %s", $auth->get_user()->username, $license_error));
		throw new Api_Error_Response('Forbidden', 403, $license_error);
	}
	$request->set_auth($auth);

	switch($request->get_project_name()) {
	case 'command':
		if (!$auth->authorized_for('api_command')) {
			throw new Api_Error_Response('Unauthorized', 401, "You're not authorized to access this page, you need 'api_command' rights");
		}
		require_once(dirname(__FILE__).'/../command.php');
		$response = generate_response($request);
		break;
	case 'config':
		if(!$auth->authorized_for('api_config')) {
			throw new Api_Error_Response('Unauthorized', 401, "You're not authorized to access this page, you need 'api_config' rights");
		}
		require_once($nacoma_path.'api/http.php');
		$response = generate_response($request);
		break;
	case 'filter':
		if(!$auth->authorized_for('api_status')) {
			throw new Api_Error_Response('Unauthorized', 401, "You're not authorized to access this page, you need 'api_status' rights");
		}
		require_once(dirname(__FILE__).'/../filter.php');
		$response = generate_response($request, $auth);
		break;
	case 'status':
		if(!$auth->authorized_for('api_status')) {
			throw new Api_Error_Response('Unauthorized', 401, "You're not authorized to access this page, you need 'api_status' rights");
		}
		require_once(dirname(__FILE__).'/../livestatus.php');
		$response = generate_response($request, $auth);
		break;
	case 'report':
		if(!$auth->authorized_for('api_report')) {
			throw new Api_Error_Response('Unauthorized', 401, "You're not authorized to access this page, you need 'api_report' rights");
		}
		require_once(dirname(__FILE__).'/../report.php');
		$response = generate_response($request, $auth);
		break;
	case 'help':
	case '':
		$response = new Api_Response('<h1>op5 Monitor API</h1><p>This is the op5 Monitor HTTP API. Each Nagios object can be queried by using the following documentation.</p>', 200);
		break;
	default:
		$response = Api_Error_Response::with_usage('Invalid request', 400, $request);
	}
} catch (Api_Response $response) {
	if($response->getCode() == 401 && $request->get_param('auth_method', 'basic_auth') == 'basic_auth') {
		header('WWW-Authenticate: Basic realm="op5 Monitor API Access"');
	}
}
echo $response->send($request);
