<?php

/**
 * @param $request Api_Request
 * @param $auth op5auth
 * @return Api_Response
 */
function generate_response(Api_Request $request, op5auth $auth) {
	$api = new Report_Api($auth);
	return $api->handle_request($request);
}

/**
 * Report section of the HTML API, façade/adapter for Ninja's report layer
 * over Merlin. Takes requests as input, leaves responses.
 */
class Report_Api
{
	private $auth;
	public $base_url; /**< The URI which all printed "/api/yadda yadda" links are based on */

	/**
	 * Array of hints to include in the web docs
	 */
	protected $troubleshooting = array(
		"slow_acl" => array(
			"title" => "How can I make the report API faster? Is there a good workflow for working with the API?",
			"content" => "Having to authorize the calling user for each host/service generates some load on the server. You can avoid the authorization performance penalty by giving your user the rights 'host_view_all' and 'service_view_all'. Note that those settings are very allowing so be careful when you're giving your users those rights. To separate concerns, consider using a dedicated report API user that you can lock down and rely on being used for this special purpose only."
		)
	);

	/**
	 * @param $auth Op5Auth
	 */
	function __construct(Op5Auth $auth) {
		$this->auth = $auth;
	}

	/**
	 * Used in a help context
	 *
	 * @return array
	 */
	function get_menu()
	{
		return array(
			'title' => _('Sections'),
			'items' => array_map(function($item) {
					$item['resource'] = preg_replace("~/api/~", "/api/help/", $item['resource']);
					return $item;
				},
				$this->sections()
			)
		);
	}

	/**
	 * "options" as in "unfiltered GET" -- let's filter those arguments here and pass them
	 * along to Summary_Reports_Model::get_events()
	 *
	 * @param $request Api_Request
	 * @return Api_Response
	 */
	function handle_event(Api_Request $request)
	{
		$options = $request->get_params();
		if(!$options) {
			return new Api_Error_Response("No options provided", 400);
		}
		if(isset($options['report_period']) && $options['report_period'] != 'custom' && (isset($options['start_time']) || isset($options['end_time']))) {
			return new Api_Error_Response(sprintf("Can not combine 'report_period' (%s) with 'start_time' or 'end_time'", $options['report_period']), 400);
		}
		load_kohana();
		$options_obj = Report_options::setup_options_obj('HttpApiEvent', $options);
		$reports = new Summary_Reports_Model($options_obj);
		$result = $reports->get_events($this->auth);
		$data = array();
		foreach($result['events'] as $row) {
			$data[] = $options_obj->to_output($row);
		}

		$options['start_time'] = $options_obj['start_time'];
		$options['end_time'] = $options_obj['end_time'];

		// let's not confuse one with the other
		$output_options = $options;
		unset($output_options['report_period']);

		ksort($output_options); // easier to remember parameters if their order is constant
		$response = array(
			'resource' => $this->base_url."event/?".http_build_query($output_options),
			'generated_at' => time(),
			'start_time' => $output_options['start_time'],
			'end_time' => $output_options['end_time'],
			'limit' => $result['limit'],
			'offset' => $result['offset']
		);
		if($result['can_paginate']) {
			// provide easy pagination through additional resource pointer (aka "link")
			$output_options['offset'] = $result['offset'] + $result['limit'];
			ksort($output_options); // easier to remember parameters if their order is constant
			$response['next_resource'] = $this->base_url."event/?".http_build_query($output_options);
			// adhere to the limit given and do the cheap thing when appreciating
			// the need for pagination: remove the extra element
			unset($data[count($data)-1]);
		}
		// make sure all options are prepended, and the data appended; keep the meta in the 'head'
		$response['data'] = $data;
		return new Api_Response($response);
	}

	/**
	 * @param $section string
	 * @param $request Api_Request
	 * @return Api_Response
	 */
	function help($section, Api_Request $request)
	{
		$sections = $this->sections();
		$response = array(
			'menu' => $this->get_menu()
		);
		switch($section) {
			case 'index':
				$response['body'] = file_get_contents(__DIR__.'/report/index.html');
				break;
			case 'event':
				load_kohana();
				$options = Report_options::setup_options_obj('HttpApiEvent');
				// @todo put all of this in HttpApiEvent_options
				$properties = $options->properties();
				$required_names = array(
					'end_time',
					'start_time'
				);
				$rows = array();
				$format_type = function($type, $enums) {
					if(!$enums) {
						$enums = array("value1", "value2");
					}
					$map = array(
						'int' => 'A number',
						'string' => 'Free text',
						'enum' => "One of ['".implode("', '", $enums)."']",
						'bool' => "1 (meaning 'yes') or 0 (meaning 'no')",
						'objsel' => "One or more names of objects",
						'array' => '',
						'timestamp' => "A Unix timestamp (it's now ".time()." for example)",
						'object' => ''
					);
					return sprintf('<abbr title="%s">%s</abbr>', $map[$type], $type == 'objsel' ? 'array' : $type);
				};
				foreach($properties as $name => $option) {
					$key = "required_$name";
					$required = true;
					if(!in_array($name, $required_names)) {
						$key = 'zzzz'.$key; // place it later than the required keys
						$required = false;
					}
					$description = null;
					if(isset($option['description'])) {
						$description = $option['description'];
					}

					$default = $option['default'];
					$options_to_print = array();
					if(isset(HttpApiEvent_options::$http_api_options[$name]) &&
						isset(HttpApiEvent_options::$http_api_options[$name]['options'])
					) {
						$http_api_options = HttpApiEvent_options::$http_api_options[$name]['options'];
						$options_to_print = array_keys($http_api_options);
						$index = array_search($option['default'], $http_api_options);
						if($index !== false) {
							$default = $index;
						}
					} elseif(isset($option['options'])) {
						$options_to_print = array_keys($option['options']);
					}
					$rows[$key] = array(
						'required' => $required,
						'name' => $name,
						'type' => $format_type($option['type'], $options_to_print),
						'default' => $options->format_default($default, $option['type'] == 'objsel' ? 'array' : $option['type']),
						'description' => $description
					);
				}

				ksort($rows);

				$live_data = sprintf(
					'<h2>Live data</h2>
					<p>List real objects from your Monitor installation in <a href="%1$sevent/?format=xml&start_time=%2$s&limit=10">XML</a>, <a href="%1$sevent/?format=json&start_time=%2$s&limit=10">JSON</a> or <a href="%1$sevent/?format=html&start_time=%2$s&limit=10">HTML</a>.</p>
					If you would like to view a single object and its properties, edit the URI to point at /api/report/?host_name=&lt;name of object here&gt;&format=xml',
					$this->base_url,
					time() - 3600 // list the latest hour's events.. I dunno :)
				);

				$example_uri = $request->get_base_url().'/api/report/event';

				$local_limit = (int) Op5Config::instance()->getConfig('http_api.report.limit');
				if(!$local_limit) {
					$local_limit = "<em>not set</em>";
				}
				$python_pagination_example = file_get_contents(__DIR__.'/examples/traverse.py');
				$example_usage = "
				<h2>Example usage</h2>
				<p>Follow the examples below to extract events.</p>

				<h3 id='example_curl'>cURL in terminal</h3>
				<div class='curl'>
					<h4>GET</h4>
					<h5>Request</h5>
					<pre><code>curl '$example_uri/?report_period=last24hours&limit=2' -u '&lt;your username&gt;:&lt;your password&gt;'</code></pre>
					<h5>Response</h5>
					<pre><samp>".'{
    "data": [
        {
            "event_type": "service_alert",
            "hard": "0",
            "host_name": "host-15",
            "in_scheduled_downtime": null,
            "output": "Lala",
            "retry": "1",
            "service_description": "service-15-18",
            "state": "unknown",
            "timestamp": "1355663326"
        },
        {
            "event_type": "service_alert",
            "hard": "1",
            "host_name": "host-34",
            "in_scheduled_downtime": null,
            "output": "Danger",
            "retry": "1",
            "service_description": "service-34-20",
            "state": "critical",
            "timestamp": "1355663326"
        }
    ],
    "end_time": 1355749724,
    "generated_at": 1355749725,
    "limit": 2,
    "next_resource": "'.$example_uri.'/?end_time=1355749724&limit=2&offset=2&start_time=1355663324",
    "offset": 0,
    "resource": "'.$example_uri.'/?end_time=1355749724&limit=2&start_time=1355663324",
    "start_time": 1355663324,
    "total": "1088466"
}'."</samp></pre>
				</div>
				<h3 id='example_python'>Python</h3>
				<div class='python'>
					<h4>GET</h4>
					<pre><code>import requests\nr = requests.get('$example_uri/?report_period=last24hours', auth=('&lt;your username&gt;', '&lt;your password&gt;'))\nprint r.text</code></pre>
				</div>

				<h3 id='example_php'>PHP with cURL</h3>
				<div class='php'>
					<h4>GET</h4>
					<pre><code>\$a_handle = curl_init('$example_uri/?report_period=last24hours');\ncurl_setopt(\$a_handle, CURLOPT_USERPWD, '&lt;your username&gt;:&lt;your password&gt;');\ncurl_setopt(\$a_handle, CURLOPT_RETURNTRANSFER, TRUE);\n\$events = curl_exec(\$a_handle);\necho \$events;</code></pre>
				</div>

				<h2>Limiting results</h2>

				<dl>
					<dt id='example_traverse'><var>?limit=10&offset=10</var></dt>
					<dd>
						<p>A hard limit which <em>might</em> be adhered to. The limit is detected in the following order:</p>
						<ol>
							<li>Absolute max limit: ".HttpApiEvent_options::MAX_EVENTS."</li>
							<li>Local op5 Monitor max limit: ".$local_limit." (can be modified in /etc/op5/http_api.yml)</li>
							<li>Your submitted <var>limit</var> GET parameter</li>
						</ol>
						<p>The complementing <var>offset</var> makes sure you can always get all of your data, by performing requests with a limit lower than the total count, and an increasing offset, until you get all of it. Here's an example python script which does this for you (you will find all examples in the folder ".__DIR__."/examples):</p>
						<div class='python'>
							<pre><code>$python_pagination_example</code></pre>
						</div>
					</dd>

					<dt><var>?report_period=last24hours</var></dt>
					<dd>
						Limit the report period to a fixed timeframe. This lets you take shortcuts if you're only interested in, say, events that occurred during the <var>last24hours</var>. All other parameters in the table above are there for you to filter events.
					</dd>
				</dl>
					";

				$response['body'] = implode("\n", array(
					$live_data,
					$example_usage
				));
				$response['description'] = $sections[$section]['description'];
				$response['properties'] = $rows;
				break;
			default:
				$response['body'] = $sections[$section]['description'];
		}
		$response['troubleshooting'] = $this->troubleshooting;
		return new Api_Response($response);
	}

	/**
	 * Main entry point for requests that should be handled by livestatus
	 *
	 * @param $request The HTTP request we're responding to
	 * @returns Api_Response
	 * @throws Api_Response
	 */
	function handle_request(Api_Request $request)
	{
		$path = $request->get_path();
		$this->base_url = $request->get_base_url().'/api/report/';
		$sections = $this->sections();
		if(!$path) {
			return $this->help('index', $request);
		}
		$section = array_shift($path);
		if($section && isset($sections[$section]) && $request->is_help()) {
			return $this->help($section, $request);
		}
		switch($section) {
			case 'event':
				return $this->handle_event($request);
			default:
				return Api_Error_Response::with_usage('Invalid request', 400, $request);
		}
	}

	/**
	 * @return array
	 */
	function sections()
	{
		return array(
			'event' => array(
				'description' => '
				<p>The report-event part of Monitor\'s API represents the <strong>extract</strong> part of <a href="https://en.wikipedia.org/wiki/Extract_transform_load" title="Extract, Transform, Load">ETL</a>:</p>
				<ol>
					<li class="highlight"><strong>Extract</strong><br />Take report data out of Monitor in XML or JSON. You get the same information that Monitor saves and displays in the GUI, e.g. what happened, when it happened, which service/host was involved.</li>
					<li><strong>Transform</strong><br />Adapt the Monitor-formatted data to fit your target destination, turning it into valid input for your external program, e.g. another report generator.</li>
					<li><strong>Load</strong><br />Use your external report program with Monitor\'s data.</li>
				</ol>
				',
				'resource' => $this->base_url.'event'
			)
		);
	}
}
